'use client';

import { Typography } from '../../typography/typography';
import { InputLayout } from '../input-layout/input-layout';
import React, { forwardRef, InputHTMLAttributes, useId, useState } from 'react';

interface SliderProps extends InputHTMLAttributes<HTMLInputElement> {
    options?: number[];
}

function fractionCalc(current: number | string, max: number | string): number {
    const _current = isNaN(parseInt(current as string))
        ? 50
        : parseInt(current as string);
    const _max = isNaN(parseInt(max as string)) ? 100 : parseInt(max as string);

    return (_current / _max) * 100;
}

function gradientStyle(value: number): string {
    return `linear-gradient( to right, var(--md-sys-color-primary) ${value}%, var(--md-sys-color-surface-container-highest) ${value}%)`;
}

export const Slider = forwardRef<HTMLInputElement, SliderProps>(
    ({ options, ...props }, ref) => {
        const sliderId = useId();
        const [isChrome, setIsChrome] = useState<boolean>(false);
        const [value, setValue] = useState(props.defaultValue ?? 50);

        useLayoutEffect(() => {
            setIsChrome(navigator.userAgent.indexOf('AppleWebKit') != -1);
        }, []);

        if (isChrome) {
            const initialFraction = fractionCalc(
                (props.value as string) ?? (props.defaultValue as string),
                props.max as string,
            );

            if (props.style === undefined) {
                props.style = {};
            }

            props.style.background = gradientStyle(initialFraction);
        }

        const webkitProgress = (event: ChangeEvent<HTMLInputElement>) => {
            if (isChrome) {
                const fraction = fractionCalc(
                    (event.target.value as string) ??
                        (event.target.defaultValue as string),
                    event.target.max,
                );

                event.target.style.background = gradientStyle(fraction);
            }
        };

        return (
            <div className={'m3 m3-slider-container'}>
                <div className={'m3 m3-slider-label'}>
                    <Typography role={'label'} size={'small'}>
                        {value}
                    </Typography>
                </div>
                <InputLayout
                    {...props}
                    className={props.className}
                    list={sliderId}
                    onChange={(event: ChangeEvent<HTMLInputElement>) => {
                        props.onChange?.apply(this, props?.onChange?.prototype);
                        setValue(event.target.value);

                        if (isChrome) {
                            webkitProgress(event);
                        }
                    }}
                    ref={ref}
                    style={{
                        ...props.style,
                        background: `linear-gradient(to right, var(--md-sys-color-primary) ${progress}%, var(--md-sys-color-surface-container-highest) ${progress}%)`,
                    }}
                    type={'range'}
                    typeInput={'slider'}
                />
                {options && (
                    <datalist id={sliderId}>
                        {options.map((option, index) => (
                            <option key={index} value={option} />
                        ))}
                    </datalist>
                )}
            </div>
        );
    },
);
